@@ -106,6 +106,7 @@ struct GcCharacterIter
   const uc_script_t * const * scripts;
   uc_general_category_t category;
   const gchar * const * keywords;
+  GcSearchFlag flags;
 
   gboolean (* filter) (GcCharacterIter *iter, ucs4_t uc);
 };
@@ -438,8 +439,24 @@ filter_keywords (GcCharacterIter *iter, ucs4_t uc)
     return FALSE;
 
   while (*keywords)
-    if (g_strstr_len (buffer, UNINAME_MAX, *keywords++) == NULL)
-      return FALSE;
+    {
+      const gchar *keyword = *keywords++;
+      size_t length = strlen (keyword);
+
+      if (length >= UNINAME_MAX)
+        return FALSE;
+
+      if (iter->flags & GC_SEARCH_FLAG_WORD)
+        {
+          if (strncmp (buffer, keyword, strlen (keyword)) != 0)
+            return FALSE;
+        }
+      else
+        {
+          if (g_strstr_len (buffer, UNINAME_MAX, keyword) == NULL)
+            return FALSE;
+        }
+    }
 
   return TRUE;
 }
@@ -522,11 +539,7 @@ gc_character_width (gunichar uc)
   return uc_width (uc, "UTF-8");
 }
 
-GQuark
-gc_search_error_quark (void)
-{
-  return g_quark_from_static_string ("gc-search-error-quark");
-}
+G_DEFINE_QUARK (gc-search-error-quark, gc_search_error)
 
 G_DEFINE_BOXED_TYPE (GcSearchResult, gc_search_result,
                      g_array_ref, g_array_unref);
@@ -661,13 +674,11 @@ struct _GcSearchContext
   enum GcSearchState state;
   GcCharacterIter iter;
   GcSearchCriteria *criteria;
+  GcSearchFlag flags;
 };
 
 struct SearchData
 {
-  GcCategory category;
-  gchar **keywords;
-  const uc_script_t **scripts;
   gunichar uc;
   gint max_matches;
   GcSearchContext *context;
@@ -676,8 +687,6 @@ struct SearchData
 static void
 search_data_free (struct SearchData *data)
 {
-  g_clear_pointer (&data->keywords, (GDestroyNotify) g_strfreev);
-  g_clear_pointer (&data->scripts, g_free);
   g_clear_object (&data->context);
   g_slice_free (struct SearchData, data);
 }
@@ -891,6 +900,7 @@ gc_character_iter_init_for_related (GcCharacterIter *iter,
 enum {
   SEARCH_CONTEXT_PROP_0,
   SEARCH_CONTEXT_PROP_CRITERIA,
+  SEARCH_CONTEXT_PROP_FLAGS,
   SEARCH_CONTEXT_LAST_PROP
 };
 
@@ -911,6 +921,9 @@ gc_search_context_set_property (GObject      *object,
     case SEARCH_CONTEXT_PROP_CRITERIA:
       context->criteria = g_value_dup_boxed (value);
       break;
+    case SEARCH_CONTEXT_PROP_FLAGS:
+      context->flags = g_value_get_flags (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -940,6 +953,11 @@ gc_search_context_class_init (GcSearchContextClass *klass)
     g_param_spec_boxed ("criteria", NULL, NULL,
                         GC_TYPE_SEARCH_CRITERIA,
                         G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE);
+  search_context_props[SEARCH_CONTEXT_PROP_FLAGS] =
+    g_param_spec_flags ("flags", NULL, NULL,
+                        GC_TYPE_SEARCH_FLAG,
+                        GC_SEARCH_FLAG_NONE,
+                        G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE);
   g_object_class_install_properties (object_class, SEARCH_CONTEXT_LAST_PROP,
                                      search_context_props);
 }
@@ -951,9 +969,13 @@ gc_search_context_init (GcSearchContext *context)
 }
 
 GcSearchContext *
-gc_search_context_new (GcSearchCriteria *criteria)
+gc_search_context_new (GcSearchCriteria *criteria,
+                       GcSearchFlag      flags)
 {
-  return g_object_new (GC_TYPE_SEARCH_CONTEXT, "criteria", criteria, NULL);
+  return g_object_new (GC_TYPE_SEARCH_CONTEXT,
+                       "criteria", criteria,
+                       "flags", flags,
+                       NULL);
 }
 
 static void
@@ -1060,6 +1082,7 @@ gc_search_context_search  (GcSearchContext    *context,
                                "search context destroyed");
       return;
     }
+  context->iter.flags = context->flags;
   g_mutex_unlock (&context->lock);
 
   data = g_slice_new0 (struct SearchData);
